@@ -8,15 +8,15 @@ How to generate ODSs from OMFITclasses, and use OMAS plot methods
 
 from matplotlib.pyplot import show
 
-from omfit.classes.omfit_eqdsk import OMFITgeqdsk, OMFITsrc
-from omfit.classes.omfit_gacode import OMFITgacode
+from omfit_classes.omfit_eqdsk import OMFITgeqdsk
+from omfit_classes.omfit_gacode import OMFITgacode
 from omas import *
 
 # =====================
 # plotting equilibrium
 # =====================
 # read gEQDSK file in OMFIT
-geq = OMFITgeqdsk(OMFITsrc + '/../samples/g133221.01000')
+geq = OMFITgeqdsk(f'{omas_dir}samples/g145419.02100')
 geq['fluxSurfaces'].load()
 
 # convert gEQDSK to OMAS data structure
@@ -32,7 +32,7 @@ omas_plot.equilibrium_summary(ods, linewidth=1)
 # plotting profiles
 # =====================
 # read input.profiles file in OMFIT
-ip = OMFITgacode(OMFITsrc + '/../samples/input.profiles')
+ip = OMFITgacode(f'{omas_dir}samples/input.profiles_145419_02100')
 
 # convert input.profiles to OMAS data structure
 ods = ip.to_omas()
