@@ -13,7 +13,7 @@ __all__ = [
     'machine_mappings',
     'load_omas_machine',
     'machine_mapping_function',
-    'run_machine_mapping_functions',
+    'test_machine_mapping_functions',
     'mdstree',
     'mdsvalue',
     'reload_machine_mappings',
@@ -30,13 +30,13 @@ _url_dir = os.sep.join([omas_rcparams['tmp_omas_dir'], 'machine_mappings', '{bra
 
 
 def python_tdi_namespace(branch):
-    '''
+    """
     Returns the namespace of the python_tdi.py file
 
     :param branch: remote branch to load
 
     :return: namespace
-    '''
+    """
     # return cached python tdi function namespace
     if branch in _python_tdi_namespace:
         return _python_tdi_namespace[branch]
@@ -64,7 +64,7 @@ def python_tdi_namespace(branch):
 
 
 def machine_to_omas(ods, machine, pulse, location, options={}, branch='', user_machine_mappings=None, cache=None):
-    '''
+    """
     Routine to convert machine data to ODS
 
     :param ods: input ODS to populate
@@ -84,7 +84,7 @@ def machine_to_omas(ods, machine, pulse, location, options={}, branch='', user_m
     :param cache: if cache is a dictionary, this will be used to establiish a cash
 
     :return: updated ODS and data before being assigned to the ODS
-    '''
+    """
     if user_machine_mappings is None:
         user_machine_mappings = {}
 
@@ -122,6 +122,7 @@ def machine_to_omas(ods, machine, pulse, location, options={}, branch='', user_m
                 cache[call] = cocosio
     elif 'COCOSIO_TDI' in mapped:
         TDI = mapped['COCOSIO_TDI'].format(**options_with_defaults)
+        treename = mapped['treename'].format(**options_with_defaults) if 'treename' in mapped else None
         cocosio = int(mdsvalue(machine, treename, pulse, TDI).raw())
 
     # CONSTANT VALUE
@@ -148,7 +149,8 @@ def machine_to_omas(ods, machine, pulse, location, options={}, branch='', user_m
             namespace.update(_namespace_mappings[idm])
             namespace['ods'] = ODS()
             namespace['__file__'] = machines(machine, branch)[:-5] + '.py'
-            tmp = compile(call, machines(machine, branch)[:-5] + '.py', 'exec')
+            printd(f"Calling `{call}` in {os.path.basename(namespace['__file__'])}", topic='machine')
+            tmp = compile(call, namespace['__file__'], 'exec')
             exec(tmp, namespace)
             ods = namespace[mapped.get('RETURN', 'ods')]
             if isinstance(cache, dict):
@@ -182,6 +184,8 @@ def machine_to_omas(ods, machine, pulse, location, options={}, branch='', user_m
 
     # transpose manipulation
     if mapped.get('TRANSPOSE', False):
+        for k in range(len(mapped['TRANSPOSE']) - len(data.shape)):
+            data = numpy.array([data])
         data = numpy.transpose(data, mapped['TRANSPOSE'])
 
     # transpose filter
@@ -194,16 +198,21 @@ def machine_to_omas(ods, machine, pulse, location, options={}, branch='', user_m
         ods[location] = data
     else:
         with omas_environment(ods, cocosio=cocosio):
-            csize = mapped.get('COORDINATES', [])
-            osize = len([c for c in mapped.get('COORDINATES', []) if c != '1...N'])
-            dsize = len(data.shape)
+            dsize = len(data.shape)  # size of the data fetched from MDS+
+            csize = len(mapped.get('COORDINATES', []))  # number of coordinates
+            osize = len([c for c in mapped.get('COORDINATES', []) if c != '1...N'])  # number of named coordinates
+            asize = location.count(':') + csize  # data size required from MDS+ to make the assignement
+            if asize != dsize:
+                raise Exception(
+                    f"Experiment data {data.shape} does not fit in `{location}` [{', '.join([':'] * location.count(':') + mapped.get('COORDINATES', []))}]"
+                )
             if dsize - osize == 0 or ':' not in location:
                 if data.size == 1:
                     data = data.item()
                 ods[location] = nanfilter(data)
             else:
-                for k in range(data.shape[0]):
-                    ods[u2n(location, [k] + [0] * 10)] = nanfilter(data[k, ...])
+                for k in itertools.product(*list(map(range, data.shape[: location.count(':')]))):
+                    ods[u2n(location, k)] = nanfilter(data[k])
 
     return ods, {'raw_data': data0, 'processed_data': data, 'cocosio': cocosio, 'branch': mappings['__branch__']}
 
@@ -215,7 +224,7 @@ _python_tdi_namespace = {}
 
 
 def machine_mappings(machine, branch, user_machine_mappings=None, return_raw_mappings=False, raise_errors=False):
-    '''
+    """
     Function to load the json mapping files (local or remote)
     Allows for merging external mapping rules defined by users.
     This function sanity-checks and the mapping file and adds extra info required for mapping
@@ -232,7 +241,7 @@ def machine_mappings(machine, branch, user_machine_mappings=None, return_raw_map
     :param raise_errors: raise errors or simply print warnings if something isn't right
 
     :return: dictionary with mapping transformations
-    '''
+    """
     if user_machine_mappings is None:
         user_machine_mappings = {}
 
@@ -248,15 +257,19 @@ def machine_mappings(machine, branch, user_machine_mappings=None, return_raw_map
         filename = machines(machine, branch)
 
         # load mappings from file following __include__ directives
-        with open(filename, 'r') as f:
-            try:
-                top = json.load(f)
-            except json.decoder.JSONDecodeError as _excp:
-                raise ValueError(f'Error reading {filename}\n' + str(_excp))
+        if not os.stat(filename).st_size:
+            top = {}
+        else:
+            with open(filename, 'r') as f:
+                try:
+                    top = json.load(f)
+                except json.decoder.JSONDecodeError as _excp:
+                    raise ValueError(f'Error reading {filename}\n' + str(_excp))
         go_deep = ['__cocos_rules__', '__options__']
         mappings = {k: {} for k in go_deep}
+        mappings.setdefault('__include__', ['_common'])
         if not return_raw_mappings:
-            for item in top.get('__include__', []):
+            for item in top.get('__include__', ['_common']):
                 include_filename = os.path.split(filename)[0] + os.sep + f'{item}.json'
                 with open(include_filename, 'r') as f:
                     try:
@@ -297,7 +310,10 @@ def machine_mappings(machine, branch, user_machine_mappings=None, return_raw_map
         _namespace_mappings[idm] = {}
         if os.path.exists(os.path.splitext(filename)[0] + '.py'):
             with open(os.path.splitext(filename)[0] + '.py', 'r') as f:
-                exec(f.read(), _namespace_mappings[idm])
+                try:
+                    exec(f.read(), _namespace_mappings[idm])
+                except Exception as _excp:
+                    raise _excp.__class__(f"Error in {os.path.splitext(filename)[0] + '.py'}\n" + str(_excp))
 
         # generate TDI for cocos_rules
         for item in mappings['__cocos_rules__']:
@@ -329,7 +345,7 @@ def machine_mappings(machine, branch, user_machine_mappings=None, return_raw_map
             if 'coordinates' in info:
                 mappings[location]['COORDINATES'] = list(map(i2o, info['coordinates']))
                 for coordinate in mappings[location]['COORDINATES']:
-                    if coordinate == '1...N':
+                    if '1...' in coordinate:
                         continue
                     elif coordinate not in mappings:
                         text = f'Missing coordinate {coordinate} for {location}'
@@ -350,7 +366,7 @@ def machine_mappings(machine, branch, user_machine_mappings=None, return_raw_map
                                     mappings[location]['COCOSIO_' + cocos_exp] = mappings['__cocos_rules__'][cocos_rule][cocos_exp]
                                     cocos_defined = True
                 if not cocos_defined:
-                    text = f'{location} must have COCOS specified'
+                    text = f'{location} must have COCOSIO specified'
                     if raise_errors:
                         raise ValueError(text)
                     else:
@@ -368,14 +384,25 @@ def machine_mappings(machine, branch, user_machine_mappings=None, return_raw_map
     return _machine_mappings[idm]
 
 
-def reload_machine_mappings():
-    '''
+def reload_machine_mappings(verbose=True):
+    """
     Flushes internal caches of machine mappings.
     This will force the mapping files to be re-read when they are first accessed.
-    '''
+
+    :param verbose: print to screen when mappings are reloaded
+    """
+    # reset machine mapping caches
     for cache in [_machine_mappings, _namespace_mappings, _python_tdi_namespace, _machines_dict, _user_machine_mappings]:
         cache.clear()
 
+    # in case users did a `from omas.machine_mappings import ...`
+    for mod in list(sys.modules):
+        if mod.startswith('omas.machine_mappings'):
+            del sys.modules[mod]
+
+    if verbose:
+        print('Reloaded OMAS machine mapping info')
+
 
 # ===================
 # list machines and update machine files
@@ -384,7 +411,7 @@ _machines_dict = {}
 
 
 def machines(machine=None, branch=''):
-    '''
+    """
     Function to get machines that have their mappings defined
     This function takes care of remote transfer the needed files (both .json and .py) if a remote branch is requested
 
@@ -394,7 +421,7 @@ def machines(machine=None, branch=''):
 
     :return: if `machine==None` returns dictionary with list of machines and their json mapping files
              if `machine` is a string, then returns json mapping filename
-    '''
+    """
 
     # return cached results
     if branch in _machines_dict:
@@ -405,7 +432,7 @@ def machines(machine=None, branch=''):
 
     # local mappings
     if not branch:
-        dir = imas_json_dir + '/../machine_mappings'
+        dir = omas_dir + 'machine_mappings'
 
     # remote mappings from GitHub
     else:
@@ -439,12 +466,12 @@ svn export --force https://github.com/gafusion/omas.git/{svn_branch}/omas/machin
     # return filename with mappings for this machine
     else:
         if machine not in _machines_dict[branch]:
-            raise NotImplementedError(f'Machine `{machine}` has no mapping defined')
+            raise NotImplementedError(f'Machine mapping file `{machine}.json` does not exist')
         return _machines_dict[branch][machine]
 
 
 def update_mapping(machine, location, value, cocosio=None, default_options=None, update_path=False):
-    '''
+    """
     Utility function that updates the local mapping file of a given machine with the mapping info of a given location
 
     :param machine: machine name
@@ -458,7 +485,7 @@ def update_mapping(machine, location, value, cocosio=None, default_options=None,
     :param update_path: use the same value for the arrays of structures leading to this location
 
     :return: dictionary with updated raw mappings
-    '''
+    """
     ulocation = l2u(p2l(location))
     value = copy.copy(value)
     if 'COORDINATES' in value:
@@ -507,16 +534,22 @@ def update_mapping(machine, location, value, cocosio=None, default_options=None,
 # ===================
 # machine mapping functions
 # ===================
-def machine_mapping_function(__all__):
+def machine_mapping_function(__regression_arguments__, **regression_args):
     """
     Decorator used to identify machine mapping functions
 
+    :param \**regression_args: arguments used to run regression test
+
     NOTE: use `inspect.unwrap(function)` to call a function decorated with `@machine_mapping_function`
           from another function decorated with `@machine_mapping_function`
     """
 
+    __all__ = __regression_arguments__['__all__']
+
     def machine_mapping_decorator(f, __all__):
         __all__.append(f.__name__)
+        if __regression_arguments__ is not None:
+            __regression_arguments__[f.__name__] = regression_args
 
         @functools.wraps(f)
         def machine_mapping_caller(*args, **kwargs):
@@ -528,18 +561,17 @@ def machine_mapping_function(__all__):
 
                 # figure out the machine name from where the function `f` is defined
                 machine = os.path.splitext(os.path.split(inspect.getfile(f))[1])[0]
-                if (
-                    machine == '<string>'
-                ):  # if `f` is called via exec then we need to look at the call stack to figure out the macchine name
+                if machine == '<string>':  # if `f` is called via exec then we need to look at the call stack to figure out the machine name
                     machine = os.path.splitext(os.path.split(inspect.getframeinfo(inspect.currentframe().f_back)[0])[1])[0]
 
                 # call signature
                 argspec = inspect.getfullargspec(f)
-                f_args_str = ", ".join('{%s!r}' % item for item in argspec.args)
+                f_args_str = ", ".join('{%s!r}' % item for item in argspec.args if not item.startswith('_'))
                 call = f"{f.__qualname__}({f_args_str})".replace('{ods!r}', 'ods').replace('{pulse!r}', '{pulse}')
                 default_options = None
                 if argspec.defaults:
                     default_options = dict(zip(argspec.args[::-1], argspec.defaults[::-1]))
+                    default_options = {item: value for item, value in default_options.items() if not item.startswith('_')}
 
             # call
             out = f(*args, **kwargs)
@@ -556,35 +588,38 @@ def machine_mapping_function(__all__):
     return lambda f: machine_mapping_decorator(f, __all__)
 
 
-def run_machine_mapping_functions(__all__, global_namespace, local_namespace):
-    '''
+def test_machine_mapping_functions(__all__, global_namespace, local_namespace):
+    """
     Function used to test python mapping functions
 
     :param __all__: list of functionss to test
 
     :param namespace: testing namespace
-    '''
+    """
+    from pprint import pprint
+
     old_OMAS_DEBUG_TOPIC = os.environ.get('OMAS_DEBUG_TOPIC', None)
-    os.environ['OMAS_DEBUG_TOPIC'] = 'mapping'
+    os.environ['OMAS_DEBUG_TOPIC'] = 'machine'
 
     # call machine mapping to make sure the json file is properly formatted
     machine = os.path.splitext(os.path.split(local_namespace['__file__'])[1])[0]
     print(f'Sanity check of `{machine}` mapping files: ... ', end='')
-    machine_mappings(machine,'',raise_errors=True)
+    machine_mappings(machine, '', raise_errors=True)
     print('OK')
 
+    __regression_arguments__ = global_namespace['__regression_arguments__']
     try:
-        from pprint import pprint
-
-        for func in __all__:
-            print('=' * len(func))
-            print(func)
-            print('=' * len(func))
+        for func_name in __all__:
+            regression_kw = {item: value for item, value in __regression_arguments__.get(func_name, {}).items() if item != '__all__'}
+            print('=' * len(func_name))
+            print(func_name)
+            pprint(regression_kw)
+            print('=' * len(func_name))
             ods = ODS()
-            func = eval(func, global_namespace, local_namespace)
+            func = eval(func_name, global_namespace, local_namespace)
             try:
                 try:
-                    func(ods)
+                    func(ods, **regression_kw)
                 except Exception:
                     raise
             except TypeError as _excp:
@@ -592,7 +627,7 @@ def run_machine_mapping_functions(__all__, global_namespace, local_namespace):
                     raise _excp.__class__(
                         str(_excp)
                         + '\n'
-                        + 'For testing purposes, make sure to provide default valuess to all arguments of the machine mapping functions'
+                        + 'For testing purposes, make sure to provide default arguments for your mapping functions via the decorator @machine_mapping_function(__regression_arguments__, ...)'
                     )
                 else:
                     raise
@@ -603,7 +638,9 @@ def run_machine_mapping_functions(__all__, global_namespace, local_namespace):
             n = max(map(lambda x: len(x), tmp))
             for item in tmp:
                 try:
-                    print(f'{item.ljust(n)}   {numpy.array(ods[item]).shape}')
+                    with warnings.catch_warnings():
+                        warnings.filterwarnings('error')
+                        print(f'{item.ljust(n)}   {numpy.array(ods[item]).shape}')
                 except Exception:
                     print(f'{item.ljust(n)}   mixed')
     finally:
@@ -616,17 +653,17 @@ def run_machine_mapping_functions(__all__, global_namespace, local_namespace):
 # ===================
 # MDS+ functions
 # ===================
-def mds_machine_to_server_mapping(server, treename):
-    '''
+def tunnel_mds(server, treename):
+    """
     Resolve MDS+ server
     NOTE: This function makes use of the optional `omfit_classes` dependency to establish a SSH tunnel to the MDS+ server.
 
-    :param machine: machine name
+    :param server: MDS+ server address:port
 
     :param treename: treename (in case treename affects server to be used)
 
     :return: string with MDS+ server and port to be used
-    '''
+    """
     try:
         import omfit_classes.omfit_mds
     except (ImportError, ModuleNotFoundError):
@@ -643,14 +680,18 @@ _mds_connection_cache = {}
 
 
 class mdstree(dict):
-    '''
+    """
     Class to handle the structure of an MDS+ tree.
     Nodes in this tree are mdsvalue objects
-    '''
+    """
 
     def __init__(self, server, treename, pulse):
         for TDI in sorted(mdsvalue(server, treename, pulse, rf'getnci("***","FULLPATH")').raw())[::-1]:
-            TDI = TDI.decode('utf8').strip()
+            try:
+                TDI = TDI.decode('utf8')
+            except AttributeError:
+                pass
+            TDI = TDI.strip()
             path = TDI.replace('::TOP', '').lstrip('\\').replace(':', '.').split('.')
             h = self
             for p in path[1:-1]:
@@ -662,24 +703,33 @@ class mdstree(dict):
 
 
 class mdsvalue(dict):
-    '''
+    """
     Execute MDS+ TDI functions
-    '''
+    """
 
     def __init__(self, server, treename, pulse, TDI, old_MDS_server=False):
-        if 'nstx' in server:
-            old_MDS_server = True
         self.treename = treename
         self.pulse = pulse
         self.TDI = TDI
-        self.old_MDS_server = old_MDS_server
+        if 'nstx' in server:
+            old_MDS_server = True
         try:
             # handle the case that server is just the machine name
-            server = machine_mappings(server, '')['__mdsserver__']
+            tmp = machine_mappings(server, '')
         except NotImplementedError:
+            # hanlde case where server is actually a URL
             if '.' not in server:
                 raise
-        self.server = mds_machine_to_server_mapping(server, self.treename)
+        else:
+            if '__mdsserver__' not in tmp or not len(tmp['__mdsserver__']):
+                raise Exception(f'Must specify `__mdsserver__` for {server}')
+            else:
+                server = tmp['__mdsserver__']
+        self.server = tunnel_mds(server, self.treename)
+        old_servers = ['skylark.pppl.gov:8500', 'skylark.pppl.gov:8501', 'skylark.pppl.gov:8000']
+        if server in old_servers or self.server in old_servers:
+            old_MDS_server = True
+        self.old_MDS_server = old_MDS_server
 
     def data(self):
         return self.raw(f'data({self.TDI})')
@@ -703,14 +753,14 @@ class mdsvalue(dict):
         return self.raw(f'size({self.TDI})')
 
     def raw(self, TDI=None):
-        '''
+        """
         Fetch data from MDS+ with connection caching
 
         :param TDI: string, list or dict of strings
             MDS+ TDI expression(s) (overrides the one passed when the object was instantiated)
 
         :return: result of TDI expression, or dictionary with results of TDI expressions
-        '''
+        """
         try:
             import time
 
@@ -718,15 +768,18 @@ class mdsvalue(dict):
             import MDSplus
 
             def mdsk(value):
-                '''
+                """
                 Translate strings to MDS+ bytes
-                '''
+                """
                 return str(str(value).encode('utf8'))
 
             if TDI is None:
                 TDI = self.TDI
 
             try:
+                out_results = None
+
+                # try connecting and re-try on fail
                 for fallback in [0, 1]:
                     if (self.server, self.treename, self.pulse) not in _mds_connection_cache:
                         conn = MDSplus.Connection(self.server)
@@ -741,46 +794,70 @@ class mdsvalue(dict):
                             del _mds_connection_cache[(self.server, self.treename, self.pulse)]
                         if fallback:
                             raise
+
+                # list of TDI expressions
                 if isinstance(TDI, (list, tuple)):
+                    TDI = {expr: expr for expr in TDI}
+
+                # dictionary of TDI expressions
+                if isinstance(TDI, dict):
+                    # old versions of MDS+ server do not support getMany
                     if self.old_MDS_server:
-                        res = {}
+                        results = {}
                         for tdi in TDI:
-                            res[tdi] = mdsvalue(self.server, self.treename, self.pulse, tdi).raw()
-                        return res
-                    else:
-                        conns = conn.getMany()
-                        for expr in TDI:
-                            conns.append(str(expr.__hash__()), expr)
-                        res = conns.execute()
-                        try:
-                            return {expr: MDSplus.Data.data(res[mdsk(expr.__hash__())][mdsk('value')]) for expr in TDI}
-                        except KeyError:
-                            return {expr: MDSplus.Data.data(res[str(expr.__hash__())][str('value')]) for expr in TDI}
-                elif isinstance(TDI, dict):
-                    if self.old_MDS_server:
-                        res = {}
-                        for tdi in TDI:
-                            res[tdi] = mdsvalue(self.server, self.treename, self.pulse, TDI[tdi]).raw()
-                        return res
+                            try:
+                                results[tdi] = mdsvalue(self.server, self.treename, self.pulse, TDI[tdi]).raw()
+                            except Exception as _excp:
+                                results[tdi] = Exception(str(_excp))
+                        out_results = results
+
+                    # more recent MDS+ server
                     else:
                         conns = conn.getMany()
                         for name, expr in TDI.items():
                             conns.append(name, expr)
                         res = conns.execute()
-                        try:
-                            return {name: MDSplus.Data.data(res[mdsk(name)][mdsk('value')]) for name, expr in TDI.items()}
-                        except KeyError:
-                            return {name: MDSplus.Data.data(res[str(name)][str('value')]) for name, expr in TDI.items()}
+                        results = {}
+                        for name, expr in TDI.items():
+                            try:
+                                results[name] = MDSplus.Data.data(res[mdsk(name)][mdsk('value')])
+                            except KeyError:
+                                try:
+                                    results[name] = MDSplus.Data.data(res[str(name)][str('value')])
+                                except KeyError:
+                                    try:
+                                        results[name] = Exception(MDSplus.Data.data(res[mdsk(name)][mdsk('error')]))
+                                    except KeyError:
+                                        results[name] = Exception(MDSplus.Data.data(res[str(name)][str('error')]))
+                        out_results = results
+
+                # single TDI expression
                 else:
-                    return MDSplus.Data.data(conn.get(TDI))
+                    out_results = MDSplus.Data.data(conn.get(TDI))
+
+                # return values
+                return out_results
+
             except Exception as _excp:
                 txt = []
                 for item in ['server', 'treename', 'pulse']:
                     txt += [f' - {item}: {getattr(self, item)}']
                 txt += [f' - TDI: {TDI}']
                 raise _excp.__class__(str(_excp) + '\n' + '\n'.join(txt))
+
         finally:
-            printd(f'{TDI} \t {time.time() - t0:3.3f} secs', topic='mapping')
+            if out_results is not None:
+                if isinstance(out_results, dict):
+                    if all(isinstance(out_results[k], Exception) for k in out_results):
+                        printd(f'{TDI} \tall NO\t {time.time() - t0:3.3f} secs', topic='machine')
+                    elif any(isinstance(out_results[k], Exception) for k in out_results):
+                        printd(f'{TDI} \tsome OK/NO\t {time.time() - t0:3.3f} secs', topic='machine')
+                    else:
+                        printd(f'{TDI} \tall OK\t {time.time() - t0:3.3f} secs', topic='machine')
+                else:
+                    printd(f'{TDI} \tOK\t {time.time() - t0:3.3f} secs', topic='machine')
+            else:
+                printd(f'{TDI} \tNO\t {time.time() - t0:3.3f} secs', topic='machine')
 
 
 # ===================
@@ -830,9 +907,9 @@ class dynamic_omas_machine(dynamic_ODS):
             return self.cache[o2u(key)][key]
 
     def __contains__(self, location):
-        ulocation = o2u(location)
         if not self.active:
             raise RuntimeError('Dynamic link broken: %s' % self.kw)
+        ulocation = o2u(location)
         if ulocation.endswith(':'):
             return False
         return ulocation in machine_mappings(self.kw['machine'], self.kw['branch'], self.kw['user_machine_mappings'])
@@ -842,13 +919,16 @@ class dynamic_omas_machine(dynamic_ODS):
         if ulocation + '.:' in machine_mappings(self.kw['machine'], self.kw['branch'], self.kw['user_machine_mappings']):
             return list(range(self[ulocation + '.:']))
         else:
-            return numpy.unique(
+            tmp = numpy.unique(
                 [
                     convert_int(k[len(ulocation) :].lstrip('.').split('.')[0])
                     for k in machine_mappings(self.kw['machine'], self.kw['branch'], self.kw['user_machine_mappings'])
-                    if k.startswith(ulocation) and len(k[len(ulocation) :].lstrip('.').split('.')[0])
+                    if not k.startswith('_') and k.startswith(ulocation) and len(k[len(ulocation) :].lstrip('.').split('.')[0])
                 ]
             )
+            if ':' in tmp:
+                raise ValueError(f"Please specify number of structures for `{o2u(location)}.:` in {self.kw['machine']}.json")
+            return tmp
 
 
 def load_omas_machine(
