@@ -345,6 +345,15 @@ def pf_active(ods, nc_weird=0, nc_undefined=0):
     return ods
 
 
+@add_to_ODS
+def ec_launchers(ods):
+    from omas import load_omas_json
+
+    pr = load_omas_json(os.path.join('omas', 'samples', 'd3d_ECRH_machine_mapping.json'))['ec_launchers']
+    ods['ec_launchers'].update(pr)
+    return ods
+
+
 @add_to_ODS
 def magnetics(ods):
     """
@@ -496,6 +505,7 @@ def gas_injection(ods):
     :return: ODS instance with fake gas injection hardware information added
     """
 
+    ods.imas_version = "3.33.0" # WARNING: the structure of gas_injection IDS changed with 3.34.0
     ods['gas_injection.time'] = [0]
 
     ods['gas_injection.pipe.0.name'] = 'FAKE_GAS_A'
