@@ -27,6 +27,7 @@ OMAS v%s only runs with Python 3.6+ and you are running Python %s
         % (__version__, '.'.join(map(str, sys.version_info[:2])))
     )
 
+import pwd
 import glob
 import json
 import copy
@@ -45,6 +46,7 @@ import traceback
 import difflib
 import weakref
 import unittest
+import itertools
 
 try:
     import tqdm
@@ -93,19 +95,24 @@ class IMAS_json_dir(str):
     pass
 
 
-imas_json_dir = IMAS_json_dir(os.path.abspath(str(os.path.dirname(__file__)) + '/imas_structures/'))
+omas_dir = os.path.abspath(str(os.path.dirname(__file__))) + os.sep
+omas_install_dir = os.path.abspath(omas_dir + os.sep + '..') + os.sep
+imas_json_dir = IMAS_json_dir(omas_dir + os.sep + 'imas_structures' + os.sep)
 
 omas_git_repo = False
-if os.path.exists(imas_json_dir + '/../../.git') and os.access(imas_json_dir + '/../../.git', os.W_OK):
+if os.path.exists(omas_install_dir + '.git') and os.access(omas_install_dir + '.git', os.W_OK):
     omas_git_repo = True
 
 
 class IMAS_versions(OrderedDict):
     """
-    dictionary with list of IMAS version and their sub-folder name in the imas_json_dir
+    Dictionary with list of IMAS version and their sub-folder name in the imas_json_dir
     """
 
     def __init__(self, mode='all'):
+        """
+        :param mode: `all`, `named`, `tagged`
+        """
         OrderedDict.__init__(self)
         if mode in ['all', 'named']:
             # first `develop/3` and other branches
@@ -117,6 +124,10 @@ class IMAS_versions(OrderedDict):
             for _item in list(map(lambda x: os.path.basename(x), sorted(glob.glob(imas_json_dir + os.sep + '*')))):
                 if _item.startswith('3'):
                     self[_item.replace('_', '.')] = _item
+        # do not include empty imas_structures directories (eg. needed to avoid issues wheen switching to old git branches)
+        for item, value in list(self.items()):
+            if not len(glob.glob(imas_json_dir + os.sep + value + os.sep + '*.json')):
+                del self[item]
 
 
 # imas versions
