@@ -21,7 +21,7 @@ def set_omas_warnings():
     warnings.filterwarnings('always', category=FutureWarning, message='The Panel class is removed.*')
     warnings.filterwarnings('always', category=UserWarning, message='Attempting to set identical left==right.*')
     warnings.filterwarnings('always', category=UserWarning, message='No contour levels were found.*')
-    warnings.filterwarnings('always', category=matplotlib.cbook.deprecation.MatplotlibDeprecationWarning)
+    warnings.filterwarnings('always', category=matplotlib.MatplotlibDeprecationWarning)
     warnings.filterwarnings('always', category=RuntimeWarning, message='invalid value encountered.*')
     warnings.filterwarnings('always', category=DeprecationWarning, message='please use dns.resolver.resolve.*')
     warnings.filterwarnings(
